/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __math_DAGNode_h__
#define __math_DAGNode_h__

namespace fe
{

#if FE_CODEGEN<=FE_DEBUG
#define FE_DAG_VALIDATE()	validate()
#else
#define FE_DAG_VALIDATE()
#endif

/**************************************************************************//**
	@brief Node in a Directed Acyclic Graph

	@ingroup math
*//***************************************************************************/
class FE_DL_EXPORT DAGNode: public Counted
{
	public:
						DAGNode(void)										{}
virtual					~DAGNode(void)										{}

		void			attachTo(sp<DAGNode> rNode);
		void			detachFrom(sp<DAGNode> rNode);
		void			detach(void);

		void			validate(void);

		List< sp<DAGNode> >::Iterator	parent(void){ return m_parents; }
		List< sp<DAGNode> >::Iterator	child(void)	{ return m_children; }

		U32				parentCount(void) const	{ return m_parents.size(); }
		U32				childCount(void) const	{ return m_children.size(); }

	private:
		List< sp<DAGNode> >	m_parents;
		List< sp<DAGNode> >	m_children;
};

inline void DAGNode::validate(void)
{
	//* verify that each parent references this node exactly once
	List< sp<DAGNode> >::Iterator iterator(m_parents);
	sp<DAGNode> spParent;
	while((spParent= *iterator++).isValid())
	{
		List< sp<DAGNode> >::Iterator iterator2(spParent->m_children);
		sp<DAGNode> spChild;
		U32 count=0;
		while((spChild= *iterator2++).isValid())
			if(spChild.raw()==this)
			{
				count++;
			}
		if(count!=1)
		{
			feX("DAGNode::validate","found %d parent references",count);
		}
	}
}

inline void DAGNode::attachTo(sp<DAGNode> spNode)
{
	FE_DAG_VALIDATE();
	ListCore::Context context;
	if(spNode->m_children.searchForElement(context,spNode))
	{
		feLog("DAGNode::attachTo node already attached\n");
		return;
	}

	spNode->m_children.append(sp<DAGNode>(this));
	m_parents.append(spNode);
	FE_DAG_VALIDATE();
}

inline void DAGNode::detachFrom(sp<DAGNode> spNode)
{
	FE_DAG_VALIDATE();
	spNode->m_children.remove(sp<DAGNode>(this));
	m_parents.remove(spNode);
	FE_DAG_VALIDATE();
}

inline void DAGNode::detach(void)
{
	FE_DAG_VALIDATE();
	List< sp<DAGNode> >::Iterator iterator(m_parents);
	sp<DAGNode> spParent;
	while((spParent= *iterator++).isValid())
	{
		detachFrom(spParent);
	}
	m_parents.removeAll();
	FE_DAG_VALIDATE();
}

} /* namespace */

#endif /* __math_DAGNode_h__ */
