import os
import sys
forge = sys.modules["forge"]

# NOTE: The Maya version of boost may be different.
# The boost-regex appears to be sensitive and they don't provide the dso.
# Moving aside their include/boost seems to fix regex,
# but it leaves their other boost dso's cross-versioned with system includes.

def setup(module):
	srcList = [	"maya.pmh",
				"MayaDraw",
				"MayaNode",
				"SurfaceAccessibleMaya",
				"SurfaceAccessorMaya",
				"SurfaceTrianglesMaya",
				"mayaDL" ]

	# force all of FE to use Maya's version of boost
#	forge.includemap['maya'] = os.path.join(os.getenv('HT'),'include')

	dll = module.DLL( "fexMayaDL", srcList )

	# this overrides previous -W flags in codegen
	forge.cppmap['codegen'] += ' -Wno-deprecated -Wall -W -Wno-parentheses -Wno-sign-compare -Wno-reorder -Wno-uninitialized -Wunused -Wno-unused-parameter'

	deplibs = forge.corelibs+ [
				"fexDrawDLLib",
				"fexOperatorDLLib",
				"fexSurfaceDLLib"	]

	maya_path = os.getenv('MAYA_ROOT')

	dll.linkmap = {}
	if forge.fe_os == "FE_LINUX":
		dll.linkmap["mayalibs"] = "-shared"
		dll.linkmap['mayalibs'] += " -L"+os.path.join(maya_path,'lib')
		dll.linkmap["mayalibs"] += " -lOpenMaya"
	elif forge.fe_os == "FE_WIN32" or forge.fe_os == "FE_WIN64":
		dll.linkmap["mayalibs"] = "TODO"

	forge.deps( ["fexMayaDLLib"], deplibs )

	if maya_path:
		module.includemap = { 'maya' : os.path.join(maya_path,'include') }

	module.Module('test')

def auto(module):
	test_file = """
#define _BOOL
#include <maya/MTypes.h>

int main(void)
{
	return(0);
}
    \n"""

	maya_path = os.getenv('MAYA_ROOT')
	if maya_path:
		forge.includemap['maya'] = os.path.join(maya_path,'include')

	return forge.cctest(test_file)
