/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

#define FE_SAC_DEBUG	FALSE

namespace fe
{

void SurfaceAttrCreateOp::initialize(void)
{
	catalog<String>("Name")="attribute";

	catalog<String>("Class")="Point";

	catalog<String>("Type")="String";

	catalog<String>("Default")="default";

	catalog<bool>("Write Values")=false;

	catalog<String>("Value");

	catalog< sp<Component> >("Input Surface");
}

void SurfaceAttrCreateOp::handle(Record& a_rSignal)
{
#if FE_SAC_DEBUG
	feLog("SurfaceAttrCreateOp::handle\n");
#endif

	const String attrName=catalog<String>("Name");
	if(attrName.empty())
	{
		feLog("SurfaceAttrCreateOp::handle no name\n");
		return;
	}
	const String attrClass=catalog<String>("Class");
	const String attrType=catalog<String>("Type");
	const String& attrDefault=catalog<String>("Default");

	//* TODO
	const BWORD attrWriteValues=catalog<bool>("Write Values");
	const String& attrValue=catalog<String>("Value");

	sp<SurfaceAccessorI> spOutputElement;
	if(!accessOutput(spOutputElement,a_rSignal,
			attrClass=="Point"? e_point: e_primitive,attrName)) return;

	I32 elementCount=spOutputElement->count();
	feLog("SurfaceAttrCreateOp::handle elementCount=%d\n",elementCount);

	if(attrType=="String")
	{
		for(I32 index=0;index<elementCount;index++)
		{
#if FE_SAC_DEBUG
			feLog("SurfaceAttrCreateOp::handle set %d \"%s\"\n",
					index,attrDefault.c_str());
#endif
			spOutputElement->set(index,0,attrDefault);
		}
	}
	else if(attrType=="I32")
	{
		const I32 value=atoi(attrDefault.c_str());
		for(I32 index=0;index<elementCount;index++)
		{
#if FE_SAC_DEBUG
			feLog("SurfaceAttrCreateOp::handle set %d %d\n",
					index,value);
#endif
			spOutputElement->set(index,0,value);
		}
	}
	else if(attrType=="Real")
	{
		const Real value=atof(attrDefault.c_str());
		for(I32 index=0;index<elementCount;index++)
		{
#if FE_SAC_DEBUG
			feLog("SurfaceAttrCreateOp::handle set %d %.6G\n",
					index,value);
#endif
			spOutputElement->set(index,0,value);
		}
	}
	else if(attrType=="SpatialVector")
	{
		const Real value=atof(attrDefault.c_str());
		const SpatialVector defaultVector(value,value,value);

		for(I32 index=0;index<elementCount;index++)
		{
#if FE_SAC_DEBUG
			feLog("SurfaceAttrCreateOp::handle set %d %s\n",
					index,c_print(defaultVector));
#endif
			spOutputElement->set(index,0,defaultVector);
		}
	}

#if FE_SAC_DEBUG
	feLog("SurfaceAttrCreateOp::handle done\n");
#endif
}

} // namespace
